$(document).ready(function() {
    // Do we really want to set a no-op ready functions here?
});

function get_extrapolation_query(host, service, track, count) {
    return {
        host: host,
        service: service,
        track: track,
        options: dataoptions[host],
        cmd: 'extrapolate'
    };
}

function fill_table(d, id) {
    var unit = d.unit ? d.unit : '';

    var tbody = $('#capacityreport-tbody-' + id);

    var tr = $('<tr>');
    $('<td>', {text: 'Data Integrity (Not NaN)'}).appendTo(tr);
    $('<td>', {text: parseFloat((d.integrity) * 100).toFixed(2) + '%'}).appendTo(tr);
    tr.appendTo(tbody);


    var tr = $('<tr>');
    $('<th>', {text: 'The Recent Past', colspan: 999}).appendTo(tr);
    tr.appendTo(tbody);

    var tr = $('<tr>');
    $('<td>', {text: 'Recent Maximum Value'}).appendTo(tr);
    $('<td>', {text: parseFloat(d.dmax).toFixed(2) + unit}).appendTo(tr);
    tr.appendTo(tbody);

    var tr = $('<tr>');
    $('<td>', {text: 'Recent Average Value'}).appendTo(tr);
    $('<td>', {text: parseFloat(d.dmean).toFixed(2) + unit}).appendTo(tr);
    tr.appendTo(tbody);


    var tr = $('<tr>');
    $('<th>', {text: 'The Extrapolated Future', colspan: 999}).appendTo(tr);
    tr.appendTo(tbody);

    var tr = $('<tr>');
    $('<td>', {text: 'Extrapolated Maximum Value'}).appendTo(tr);
    $('<td>', {text: parseFloat(d.emax).toFixed(2) + unit}).appendTo(tr);
    tr.appendTo(tbody);

    var tr = $('<tr>');
    $('<td>', {text: 'Extrapolated Average Value'}).appendTo(tr);
    $('<td>', {text: parseFloat(d.emean).toFixed(2) + unit}).appendTo(tr);
    tr.appendTo(tbody);


    var tr = $('<tr>');
    $('<th>', {text: 'Past vs Future Analysis', colspan: 999}).appendTo(tr);
    tr.appendTo(tbody);

    var delta = d.emean - d.dmean;
    var sign = (delta < 0) ? '-' : '+';
    var text = sign + parseFloat(Math.abs(delta)).toFixed(2) + unit;
    var pct = delta/d.dmean * 100;
    if (isFinite(pct)) text += ' (' + sign + parseFloat(Math.abs(pct)).toFixed(2) + '%)';

    var tr = $('<tr>');
    $('<td>', {text: 'Average will change'}).appendTo(tr);
    $('<td>', {text: text}).appendTo(tr);
    tr.appendTo(tbody);

    var delta = d.emax - d.dmax;
    var sign = (delta < 0) ? '-' : '+';
    var text = sign + parseFloat(Math.abs(delta)).toFixed(2) + unit;
    var pct = delta/d.dmax * 100;
    if (isFinite(pct)) text += ' (' + sign + parseFloat(Math.abs(pct)).toFixed(2) + '%)';

    var tr = $('<tr>');
    $('<td>', {text: 'Maximum will change'}).appendTo(tr);
    $('<td>', {text: text}).appendTo(tr);
    tr.appendTo(tbody);


    /** The following is code for the value at data functionality which 
     * has been removed for now. It remains here as scar tissue in order to
     * get started on it again in the future.
     *
     * var tr = $('<tr>');
     * $('<th>', {text: 'Date Value Projections', colspan: 999}).appendTo(tr);
     * tr.appendTo(tbody);
     *  
     * $.each(d.dates, function(i, d) {
     * var date = d.date;
     * var value = d.value;
     * var tr = $('<tr>');
     * $('<td>', {text: date}).appendTo(tr);
     * $('<td>', {text: value}).appendTo(tr);
     * tr.appendTo(tbody);
     * });
     */
}

function make_highcharts_constant_line(name, color, level, start, step, stop) {
    var n = Math.floor(stop - start) / step;
    var d = new Array(n);
    for (var i = 0; i < n; ++i) d[i] = level;

    return {
        type: 'line',
        color: color,
        name: name,
        pointStart: start,
        pointInterval: step,
        visible: false,
        data: d
    }
}

function make_highcharts_fit_range(fit, name, delta, zIndex, fillOpacity, fillColor) {
    var n = fit.data.length;
    var d = new Array(n);
    for (var i = 0; i < n; ++i) d[i] = [
        fit.pointStart + i * fit.pointInterval,
        fit.data[i] - delta,
        fit.data[i] + delta
    ];

    var series = {
        type: 'arearange',
        name: name,
        pointStart: fit.pointStart,
        pointInterval: fit.pointInterval,
        linked: 'previous',
        zIndex: zIndex,
        fillOpacity: fillOpacity,
        data: d
    };

    if (fillColor) series.fillColor = fillColor;

    return series;
}

function make_highcharts(data, host, service, track, uid, width, height) {
    var title = track + ' for ' + service + ' on ' + host;

    var hideoptions = (get_url_parameter('hideoptions') == '1');
    var tracking = !hideoptions;

    var series_data = data.highcharts;

    var fit;
    $.each(series_data, function(index, series) {
        // Cache the fit line series for later...
        if (series.name == 'Fit') fit = series;
        // Change area series to line, and set arearange fill to white
        // when asked to hideoptions for PDF reports.
        if (hideoptions) {
            if (series.type == 'arearange') {
                series.fillColor = 'white';
            } else if (series.type == 'area') {
                series.type = 'line';
            }
        }
    });

    // Add arearange series for one and two adjusted sigmas around the fit.
    if (fit) {
        var fillColor = hideoptions ? 'white' : null;
        series_data.push(make_highcharts_fit_range(
            fit, '+/- 1 SE', 1*data.adjusted_sigma, -1, 0.1, fillColor
        ));
        series_data.push(make_highcharts_fit_range(
            fit, '+/- 2 SE', 2*data.adjusted_sigma, -2, 0.1, fillColor
        ));
    }

    // Add warning and critical level lines if not hiding options, and we
    // have first and last timepoints and critical or warning level values.
    var start = data.t_start * 1000;
    var step = data.t_step * 1000;
    var stop = data.t_stop * 1000;
    if (!hideoptions && start && step && stop) {
        if (data.warn_level !== undefined) {
            series_data.push(make_highcharts_constant_line(
                'Warning', '#dd0', data.warn_level, start, step, stop
            ));
        }
        if (data.crit_level !== undefined) {
            series_data.push(make_highcharts_constant_line(
                'Critical', '#d00', data.crit_level, start, step, stop
            ));
        }
    }

    Highcharts.setOptions({
        global: {
            useUTC: false
        }
    });

    var yAxis = {
        title: {
            text: track
        },
        min: 0, // @todo: Revisit to support negative values.
        startOnTick: false,
        showFirstLabel: false
    };
    if (data.unit) yAxis.title.text += ' (' + data.unit + ')';
    if (data.dmax == 0 && data.emax == 0) {
        // Fix the y-axcis labels and limits for all zero data.
        // @todo: We should check min values too to support negative values.
        yAxis.max = 1;
        yAxis.allowDecimals = false;
        yAxis.showFirstLabel = true;
        yAxis.showLastLabel = false;
    }

    var chart = new Highcharts.Chart({
        chart: {
            renderTo: 'highcharts-target-' + uid,
            zoomType: 'x',
            spacingRight: 20,
            height: height,
            width: width,
            animation: false,
            ignoreHiddenSeries: true
        },
        credits: {
            enabled: false
        },
        title: {
            text: title
        },
        xAxis: {
            type: 'datetime',
            maxZoom: 300000,
            title: {
                text: null
            }
        },
        yAxis: yAxis,
        tooltip: {
            enabled: tracking,
            crosshairs: true,
            shared: true,
            shape: 'callout',
            valueDecimals: 3,
            valueSuffix: data.unit ? ' ' + data.unit : '',
            xDateFormat: '%Y-%m-%d %H:%M'
        },
        legend: {
            enabled: true
        },
        plotOptions: {
            series: {
                fillOpacity: 0.5,
                animation: false,
                shadow: false,
                enableMouseTracking: tracking
            },
            area: {
                lineWidth: 1,
                marker: {
                    enabled: false,
                    states: {
                        hover: {
                            enabled: true,
                            radius: 4
                        }
                    }
                },
                shadow: false,
                states: {
                    hover: {
                        lineWidth: 1.5
                    }
                }
            },
            arearange: {

            },
            line: {
                lineWidth: 1.5,
                marker: {
                    enabled: false,
                    states: {
                        hover: {
                            enabled: true,
                            radius: 4
                        }
                    }
                },
                shadow: false,
                states: {
                    hover: {
                        lineWidth: 2
                    }
                }
            }

        },
        series: series_data
    });

    if (data.error) {
        chart.showLoading('No data to display: ' + data.error);
    }
}

function get_url_parameter(name) {
    var vars = window.location.search.substring(1).split('&');
    for (var i = 0; i < vars.length; ++i) {
        var v = vars[i].split('=');
        if (v[0] == name) return unescape(v[1]);
    }
    return null;
}
